package milab;

import javax.xml.bind.*;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Schema;
import javax.xml.transform.stream.StreamSource;
import org.xml.sax.SAXException;
import java.io.*;

public class JAXBUnMarshaller {
	
	public Object readXMLDocument(String filePath, String schemaPath, String contextPackage) {
		
		Object ret = new Object();
		
		try {

			File xmlDocument = new File(filePath);
			JAXBContext jaxbContext = JAXBContext.newInstance(contextPackage);
			Unmarshaller unMarshaller = jaxbContext.createUnmarshaller();
			SchemaFactory schemaFactory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
			Schema schema = schemaFactory.newSchema(new File(schemaPath));
			unMarshaller.setSchema(schema);
			CustomValidationEventHandler validationEventHandler = new CustomValidationEventHandler();
			unMarshaller.setEventHandler(validationEventHandler);

			ret = unMarshaller.unmarshal(xmlDocument);
			
		} catch (JAXBException e) {
			
			System.out.println(e.toString());
			
		} catch (SAXException e) {
			
			System.out.println(e.toString());
			
		}
		
		return ret;
		
	}

	public Object readXMLString(String xmlString, String schemaPath, String contextPackage) {

		Object ret = new Object();
		
		try {

			JAXBContext jaxbContext = JAXBContext.newInstance(contextPackage);
			Unmarshaller unMarshaller = jaxbContext.createUnmarshaller();
			SchemaFactory schemaFactory = SchemaFactory.newInstance("http://www.w3.org/2001/XMLSchema");
			Schema schema = schemaFactory.newSchema(new File(schemaPath));
			unMarshaller.setSchema(schema);
			CustomValidationEventHandler validationEventHandler = new CustomValidationEventHandler();
			unMarshaller.setEventHandler(validationEventHandler);

			ret = unMarshaller.unmarshal(new StreamSource(new StringReader(xmlString)));
			
		} catch (JAXBException e) {
			
			System.out.println(e.toString());
			
		} catch (SAXException e) {
			
			System.out.println(e.toString());
			
		}

		return ret;
		
	}
	
	class CustomValidationEventHandler implements ValidationEventHandler {
		
		public boolean handleEvent(ValidationEvent event) {
			
			if (event.getSeverity() == ValidationEvent.WARNING) {
				
				return true;
			}
			
			if ((event.getSeverity() == ValidationEvent.ERROR) || (event.getSeverity() == ValidationEvent.FATAL_ERROR)) {

				System.out.println("Validation Error:" + event.getMessage());
				ValidationEventLocator locator = event.getLocator();
				System.out.println("at line number:" + locator.getLineNumber());
				System.out.println("Unmarshalling Terminated");
				return false;
				
			}
			
			return true;
			
		}

	}
	
}